#include "stdafx.h"
#include "..\..\DuiLib\UIlib.h"
#include "UIFriends.hpp"
#include "main_frame.hpp"
#include "NeMessengerDlg.h"
#include <algorithm>

namespace DuiLib
{

CFriendsUI::CFriendsUI(CPaintManagerUI& paint_manager)
	: root_node_(NULL)
	, delay_deltaY_(0)
	, delay_number_(0)
	, delay_left_(0)
	, level_text_start_pos_(0)
	, text_padding_(0, 0, 0, 0)
	, paint_manager_(paint_manager)
{
	m_nItemNormalWidth = 47;
	m_nItemNormalHeight = 47;
	m_nItemSelectedWidth = 47;
	m_nItemSelectedHeight = 47;
	SetItemShowHtml(true);

	root_node_ = new Node;
	root_node_->data().level_ = -1;
	root_node_->data().child_visible_ = true;
	root_node_->data().has_child_ = true;
	root_node_->data().list_elment_ = NULL;
	m_bSingleSelect = false;
	m_bSingleControlSelect = false;
	
}

CFriendsUI::~CFriendsUI()
{
	if (root_node_)
		delete root_node_;

	root_node_ = NULL;
}

bool CFriendsUI::Add(CControlUI* pControl)
{
	if (!pControl)
		return false;

	if (_tcsicmp(pControl->GetClass(), _T("ListContainerElementUI")) != 0)
		return false;

	return CListUI::Add(pControl);
}

bool CFriendsUI::AddAt(CControlUI* pControl, int iIndex)
{
	if (!pControl)
		return false;

	if (_tcsicmp(pControl->GetClass(), _T("ListContainerElementUI")) != 0)
		return false;

	return CListUI::AddAt(pControl, iIndex);
}

bool CFriendsUI::Remove(CControlUI* pControl)
{
	if (!pControl)
		return false;

	if (_tcsicmp(pControl->GetClass(), _T("ListContainerElementUI")) != 0)
		return false;

	if (reinterpret_cast<Node*>(static_cast<CListContainerElementUI*>(pControl->GetInterface(_T("ListContainerElement")))->GetTag()) == NULL)
		return CListUI::Remove(pControl);
	else
		return RemoveNode(reinterpret_cast<Node*>(static_cast<CListContainerElementUI*>(pControl->GetInterface(_T("ListContainerElement")))->GetTag()));
}

bool CFriendsUI::RemoveAt(int iIndex)
{
	CControlUI* pControl = GetItemAt(iIndex);
	if (!pControl)
		return false;

	if (_tcsicmp(pControl->GetClass(), _T("ListContainerElementUI")) != 0)
		return false;

	if (reinterpret_cast<Node*>(static_cast<CListContainerElementUI*>(pControl->GetInterface(_T("ListContainerElement")))->GetTag()) == NULL)
		return CListUI::RemoveAt(iIndex);
	else
		return RemoveNode(reinterpret_cast<Node*>(static_cast<CListContainerElementUI*>(pControl->GetInterface(_T("ListContainerElement")))->GetTag()));
}

void CFriendsUI::RemoveAll()
{
	CListUI::RemoveAll();
	int nCount = root_node_->num_children();
	for (int i = nCount - 1; i >=0;  i--)
	{
		Node* child = root_node_->child(i);
		RemoveNode(child);
	}
	delete root_node_;

	root_node_ = new Node;
	root_node_->data().level_ = -1;
	root_node_->data().child_visible_ = true;
	root_node_->data().has_child_ = true;
	root_node_->data().list_elment_ = NULL;
}

void CFriendsUI::DoEvent(TEventUI& event) 
{
	if (!IsMouseEnabled() && event.Type > UIEVENT__MOUSEBEGIN && event.Type < UIEVENT__MOUSEEND)
	{
		if (m_pParent != NULL)
			m_pParent->DoEvent(event);
		else
			CVerticalLayoutUI::DoEvent(event);
		return;
	}

	if (event.Type == UIEVENT_TIMER && event.wParam == SCROLL_TIMERID)
	{
		if (delay_left_ > 0)
		{
			--delay_left_;
			SIZE sz = GetScrollPos();
			LONG lDeltaY =  (LONG)(CalculateDelay((double)delay_left_ / delay_number_) * delay_deltaY_);
			if ((lDeltaY > 0 && sz.cy != 0)  || (lDeltaY < 0 && sz.cy != GetScrollRange().cy ))
			{
				sz.cy -= lDeltaY;
				SetScrollPos(sz);
				return;
			}
		}
		delay_deltaY_ = 0;
		delay_number_ = 0;
		delay_left_ = 0;
		m_pManager->KillTimer(this, SCROLL_TIMERID);
		return;
	}
	if (event.Type == UIEVENT_SCROLLWHEEL)
	{
		LONG lDeltaY = 0;
		if (delay_number_ > 0)
			lDeltaY =  (LONG)(CalculateDelay((double)delay_left_ / delay_number_) * delay_deltaY_);
		switch (LOWORD(event.wParam))
		{
		case SB_LINEUP:
			if (delay_deltaY_ >= 0)
				delay_deltaY_ = lDeltaY + 8;
			else
				delay_deltaY_ = lDeltaY + 12;
			break;
		case SB_LINEDOWN:
			if (delay_deltaY_ <= 0)
				delay_deltaY_ = lDeltaY - 8;
			else
				delay_deltaY_ = lDeltaY - 12;
			break;
		}
		if
			(delay_deltaY_ > 100) delay_deltaY_ = 100;
		else if
			(delay_deltaY_ < -100) delay_deltaY_ = -100;

		delay_number_ = (DWORD)sqrt((double)abs(delay_deltaY_)) * 5;
		delay_left_ = delay_number_;
		m_pManager->SetTimer(this, SCROLL_TIMERID, 50U);
		return;
	}
	if(event.Type == UIEVENT_RBUTTONUP)
	{
		m_pManager->SendNotify(this, DUI_MSGTYPE_ITEMCLICK, UIEVENT_RBUTTONUP, -1);
	}

	CListUI::DoEvent(event);
}

Node* CFriendsUI::GetRoot()
{
	return root_node_;
}

const TCHAR* const kLogoButtonControlName = _T("logo");
const TCHAR* const kLogoContainerControlName = _T("logo_container");
const TCHAR* const kNickNameControlName = _T("nickname");
const TCHAR* const kArrowIconControlName = _T("arrowicon");
const TCHAR* const kDescriptionControlName = _T("description");
const TCHAR* const kOperatorPannelControlName = _T("operation");

static bool OnLogoButtonEvent(void* event) 
{
	TEventUI *pEvent = (TEventUI*)event;

    if( ((TEventUI*)event)->Type == UIEVENT_BUTTONDOWN ) 
	{
		CControlUI* pButton = pEvent->pSender;
		if( pButton != NULL ) 
		{
			CString name = pButton->GetName();
			if ( 0 == _tcsicmp(pButton->GetName(),_T("iptbtn")))
			{
				if(GET_APP()->IsAvailableIPT())
				{
					CListContainerElementUI* pListElement = (CListContainerElementUI*)(pButton->GetTag());
					if (pListElement)
					{
                       Node* pNode = (Node*)pListElement->GetTag();
					   if (pNode != NULL)
					   {
						   NTIPTStatusType iptMyStatus = g_pMessengerActor->GetMyIptStatus();
						   NTIPTStatusType iptStatusOfBuddy =GET_MESSENGER_DLG()->GetMainFrame()->GetMemberIPTStatus(pNode->data().value);
						 //  const NE_MEMBER_DATA MembereData = GET_MESSENGER_DLG()->GetWorkGroupController()->m_pModel->GetMember(pNode->data().value);
						   const NE_MEMBER_DATA MembereData = pNode->member_data();
						   CMenu menuList;
						   menuList.LoadMenu(IDR_MEMU_COMMON_MENU);
						   CMenu* pPopup = menuList.GetSubMenu(7);
						   GET_MESSENGER_DLG()->GetMainFrame()->MakeIPTMenu(iptMyStatus, iptStatusOfBuddy, &MembereData, pPopup);

						   if( pPopup != NULL )
						   {
							   CPoint point(pEvent->ptMouse.x, pEvent->ptMouse.y);
							   ::ClientToScreen(GET_MESSENGER_DLG()->GetMainFrame()->GetSafeHwnd(), &point);
							   int nCmdID = pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RETURNCMD | TPM_RIGHTBUTTON, point.x, point.y, CWnd::FromHandle(GET_MESSENGER_DLG()->GetMainFrame()->GetSafeHwnd()));
							   GET_MESSENGER_DLG()->GetMainFrame()->OnMenuIPT(&MembereData, nCmdID);
						   }
					   }
					}
			
				}
			} 
			else
			{	
				CListContainerElementUI* pListElement = (CListContainerElementUI*)(pButton->GetTag());
				if( pListElement != NULL ) 
				{
					CFriendsUI *pThis = (CFriendsUI *)pListElement->GetOwner();
					CButtonUI* logo_button = static_cast<CButtonUI*>(pThis->paint_manager_.FindSubControlByName(pListElement, kLogoButtonControlName));
					if (logo_button!=NULL)
					{
						CRect rcItem = pButton->GetPos();

						Node* pNode = (Node*)pListElement->GetTag();
						if (pNode != NULL)
						{
							RECT rcClient = {0};
							GetWindowRect(GET_MESSENGER_DLG()->GetMainFrame()->GetSafeHwnd(), &rcClient);
							RECT rcAround = {rcClient.left + rcItem.left - 6, rcClient.top + rcItem.top, rcClient.left + rcItem.right, rcClient.top + rcItem.bottom};

							if (GET_CONFIG()->EnableFunction(_T("UI_03")))
							{
								GET_MESSENGER_DLG()->GetProfileDlgController()->CreateProfileDlg(pNode->data().value.GetData(), rcAround, GET_MESSENGER_DLG()->GetMainFrame()->GetSafeHwnd());
							}
						}
					}
				}
			}	
		}
    }
	else if( ((TEventUI*)event)->Type == UIEVENT_MOUSEENTER  || ((TEventUI*)event)->Type == UIEVENT_MOUSELEAVE )
	{
		CControlUI* pButton = pEvent->pSender;
		if( pButton != NULL ) 
		{
			CListContainerElementUI* pListElement = (CListContainerElementUI*)(pButton->GetTag());
            if( pListElement != NULL )
			{
				CFriendsUI *pThis = (CFriendsUI *)pListElement->GetOwner();

				if (1)
				{
					CLabelUI* profile = static_cast<CLabelUI*>(pThis->paint_manager_.FindSubControlByName(pListElement, _T("profile")));
					if(profile != NULL)
					{
						BOOL bGroupChat = (BOOL)profile->GetTag();
						if (!bGroupChat)
						{
							TCHAR szBuf[MAX_PATH] = {};
							_stprintf_s(szBuf, MAX_PATH - 1, _T("%s"), _T("skin\\nTalkRes\\img_main_chat_list_profile_bar.png"));
							if(((TEventUI*)event)->Type == UIEVENT_MOUSEENTER)
							{
								profile->SetBkImage(szBuf);
							}
							else
							{
								profile->SetBkImage(_T(""));
							}
						}
						
					}

					CButtonUI* logo = static_cast<CButtonUI*>(pThis->paint_manager_.FindSubControlByName(pListElement, _T("logo")));
					if(logo != NULL)
					{
						TCHAR szBuf[MAX_PATH] = {};
						if(((TEventUI*)event)->Type == UIEVENT_MOUSEENTER)
						{
							_stprintf_s(szBuf, MAX_PATH - 1, _T("%s"), _T("skin\\nTalkRes\\img_main_chat_list_profile_sel.png"));
						}
						else
						{
							_stprintf_s(szBuf, MAX_PATH - 1, _T("%s"), _T("skin\\nTalkRes\\img_main_chat_list_profile_nor.png"));
						}
						logo->SetForeImage(szBuf);
					}
				}
			}
		}
	}

    return true;
}

Node* CFriendsUI::GetNodeByIndex(int index){
  ASSERT(root_node_);
  return root_node_->child(index);

}

Node* CFriendsUI::GetChildNodeByStrId(Node* pNode,CDuiString strID){
	ASSERT(pNode);
	for (int i = 0; i < pNode->num_children(); i++)
	{
		if(pNode->child(i)->data().value.CompareNoCase(strID) == 0)
		
			return pNode->child(i);
		
	}

	return NULL;
}


Node* CFriendsUI::FindNodeByCompareData(int nCpr)
{
	for (int i = 0; i < root_node_->num_children(); i++)
	{
		if(root_node_->child(i)->GetCompareData() == nCpr)
		{
			return root_node_->child(i);
		}
	}
		
	return NULL;
}

Node* CFriendsUI::AddChattingRoomNode(const FriendListItemInfo& item, int nCompareData, Node* parent)
{
	if (!parent)
		parent = root_node_;

	CString strBuf = _T("");

	CListContainerElementUI* pListElement = NULL;
	if( !m_dlgBuilder.GetMarkup()->IsValid() ) {
		pListElement = static_cast<CListContainerElementUI*>(m_dlgBuilder.Create(_T("chatRoom_list_item.xml"), (UINT)0, NULL, &paint_manager_));
	}
	else {
		pListElement = static_cast<CListContainerElementUI*>(m_dlgBuilder.Create((UINT)0, &paint_manager_));
	}
	if (pListElement == NULL)
		return NULL;

	//pListElement->SetCompareData(nCompareData);

	Node* node = new Node;

	node->SetComapreData(nCompareData);

	node->data().level_ = parent->data().level_ + 1;
	if (item.folder)
		node->data().has_child_ = !item.empty;
	else
		node->data().has_child_ = false;

	node->data().folder_ = item.folder;

	node->data().child_visible_ = (node->data().level_ == 0);
	node->data().child_visible_ = false;

	node->data().text_ = item.nick_name;
	node->data().value = item.id;
	node->data().list_elment_ = pListElement;

	if (!parent->data().child_visible_)
		pListElement->SetVisible(false);

	if (parent != root_node_ && !parent->data().list_elment_->IsVisible())
		pListElement->SetVisible(false);

	CDuiRect rcPadding = text_padding_;
	for (int i = 0; i < node->data().level_; ++i)
	{
		rcPadding.left += level_text_start_pos_;		
	}
	pListElement->SetPadding(rcPadding);

	CButtonUI* log_button = static_cast<CButtonUI*>(paint_manager_.FindSubControlByName(pListElement, kLogoButtonControlName));
	if (log_button != NULL)
	{
		if (!item.folder && !item.logo.IsEmpty())
		{
			strBuf = item.logo;
			log_button->SetBkImage(strBuf);
		}
		else
		{
			CContainerUI* logo_container = static_cast<CContainerUI*>(paint_manager_.FindSubControlByName(pListElement, kLogoContainerControlName));
			if (logo_container != NULL)
				logo_container->SetVisible(false);
		}
		log_button->SetTag((UINT_PTR)pListElement);
		log_button->OnEvent += MakeDelegate(&OnLogoButtonEvent);
	}

	CDuiString html_text = _T("");

	strBuf = item.nick_name;
	html_text += strBuf;

	CLabelUI* nick_name = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(pListElement, kNickNameControlName));
	if (nick_name != NULL)
	{
		if (item.folder)
			nick_name->SetFixedWidth(0);

		//nick_name->SetShowHtml(true);
		nick_name->SetText(html_text);
		if(!item.folder)
		{
			nick_name->SetTextColor(0xFF000000);
		}
	}

	strBuf = item.description;
	html_text = strBuf;

	CLabelUI* lastmsg = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(pListElement, _T("lastmsg")));
	if (lastmsg != NULL)
	{
		if (item.folder)
			lastmsg->SetFixedWidth(0);

		//lastmsg->SetShowHtml(true);
		lastmsg->SetText(html_text);
	}

	pListElement->SetFixedHeight(58);
	if(item.folder)pListElement->SetFixedHeight(29);
	pListElement->SetTag((UINT_PTR)node);

	if (!CListUI::AddAt(pListElement, 0))
	{
		delete pListElement;
		delete node;
		node = NULL;
	}

	parent->insert_child(node, 0);
	return node;
}

Node* CFriendsUI::AddNodeToFavoriteList(const FriendListItemInfo& item, int nCompareData, Node* parent) // Add group/buddy to favorite list.
{
	if (!parent)
		parent = root_node_;

	TCHAR szBuf[MAX_PATH] = {0};

	CListContainerElementUI* pListElement = NULL;
	if( !m_dlgBuilder.GetMarkup()->IsValid() ) {
		pListElement = static_cast<CListContainerElementUI*>(m_dlgBuilder.Create(_T("friend_list_item.xml"), (UINT)0, NULL, &paint_manager_));
	}
	else {
		pListElement = static_cast<CListContainerElementUI*>(m_dlgBuilder.Create((UINT)0, &paint_manager_));
	}
	if (pListElement == NULL)
		return NULL;

	CDuiString strTooltip;
	if (!item.folder)
	{
		const IGTBuddyUser * pBuddyUser = g_pMessengerActor->GetBuddyInfo(item.id);
		if(pBuddyUser == NULL)
		{
			pBuddyUser = g_pMessengerActor->GetBuddyInfoFromPersist(item.id);
		}
        ASSERT(pBuddyUser);
		if (pBuddyUser)
		{
			const NE_MEMBER_DATA* data = g_pMessengerActor->GetEmployeeInfo(item.id);
			GTUserStatusType stStatus = pBuddyUser->GetStatus();

 			CDuiString strStatus;
// 			if (ustOFFLINE == stStatus && GET_CONFIG()->EnableFunction(_T("UI_01")))
// 			{
// 				strTooltip = data->strWorkStatus.c_str();
// 			}
// 			else
			{
				strStatus = GetPresenceTip(stStatus);
				CDuiString fmt = LoadStringFromResource(IDS_STRING_MAIN_TOOLTIP_13);
				strTooltip.Format(fmt,item.nick_name.GetData(),strStatus.GetData(),data == NULL?NULL:data->strEmail.c_str());
			}

		}

	}

	//pListElement->SetCompareData(nCompareData);

	Node* node = new Node;

	node->SetComapreData(nCompareData);

	node->data().level_ = parent->data().level_ + 1;
	if (item.folder)
		node->data().has_child_ = !item.empty;
	else
		node->data().has_child_ = false;

	node->data().folder_ = item.folder;

	node->data().child_visible_ = (node->data().level_ == 0);
	node->data().child_visible_ = false;

	node->data().text_ = item.nick_name;
	node->data().value = item.id;
	node->data().list_elment_ = pListElement;

	if (!parent->data().child_visible_)
		pListElement->SetVisible(false);

	//if (parent != root_node_ && !parent->data().list_elment_->IsVisible())
	//	pListElement->SetVisible(false);

	CDuiRect rcPadding = text_padding_;
	for (int i = 0; i < node->data().level_; ++i)
	{
		rcPadding.left += level_text_start_pos_;		
	}
	pListElement->SetPadding(rcPadding);

	CLabelUI* pLabProfile = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(pListElement, _T("profile")));
	if (pLabProfile != NULL)
	{
		BOOL bGroupChat = FALSE;
		pLabProfile->SetTag((UINT_PTR)bGroupChat);
	}

	CLabelUI* log_button = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(pListElement, kLogoButtonControlName));
	if (log_button != NULL)
	{
		log_button->SetToolTip(strTooltip);
		if (!item.folder && !item.logo.IsEmpty())
		{
#if defined(UNDER_WINCE)
			_stprintf(szBuf, _T("%s"), item.logo);
#else
			_stprintf_s(szBuf, MAX_PATH - 1, _T("%s"), item.logo);
#endif
			log_button->SetBkImage(szBuf); 
		}
		else
		{
			CContainerUI* logo_container = static_cast<CContainerUI*>(paint_manager_.FindSubControlByName(pListElement, kLogoContainerControlName));
			if (logo_container != NULL)
				logo_container->SetVisible(false);
		}
		log_button->SetTag((UINT_PTR)pListElement);
		log_button->OnEvent += MakeDelegate(&OnLogoButtonEvent);
	}

	CDuiString html_text = _T("");

	CLabelUI* arrowIcon = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(pListElement, kArrowIconControlName));
	CDuiString strName = arrowIcon->GetName();
	if (node->data().has_child_){
		arrowIcon->SetVisible(true);
		if (node->data().child_visible_)
			arrowIcon->SetBkImage(_T("img_main_favorite_arrowup_nor.png"));
		else
			arrowIcon->SetBkImage(_T("img_main_favorite_arrowdown_nor.png"));
	}
	else{
		arrowIcon->SetVisible(false);
	}

	if (item.folder)
	{
		html_text += node->data().text_;
	}
	else
	{
#if defined(UNDER_WINCE)
		_stprintf(szBuf, _T("%s"), item.nick_name);
#else
		_stprintf_s(szBuf, MAX_PATH - 1, _T("%s"), item.nick_name);
#endif
		html_text += szBuf;
	}

	int nDisplaType = GET_CONFIG()->GetNameDisplayType();

	CLabelUI* nick_name = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(pListElement, kNickNameControlName));
	CLabelUI* nick_span = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(pListElement, _T("nick_span")));
	if (nick_name != NULL)
	{
		if (item.folder)
			nick_name->SetFixedWidth(0);

		nick_name->SetShowHtml(true);
		nick_name->SetText(html_text);

		if(!item.folder)
		{
			nick_name->SetToolTip(strTooltip);
			if((0 == nDisplaType || 1 ==nDisplaType))
			{
				nick_name->SetVisible(true);
			}
			else
			{
				nick_name->SetVisible(true);
			}

			nick_span->SetVisible(true);
			nick_name->SetTextColor(0xFF000000);
		}
	}

	if (!item.folder && !item.description.IsEmpty())
	{
		CHorizontalLayoutUI* mobile = static_cast<CHorizontalLayoutUI*>(paint_manager_.FindSubControlByName(pListElement, _T("mobile_online")));	
		if (item.mobile)
		{
			if (mobile != NULL)
			{

				mobile->SetVisible(false);

			}
		}
	}

	if(!item.folder)
	{
		CHorizontalLayoutUI* list_item = static_cast<CHorizontalLayoutUI*>(paint_manager_.FindSubControlByName(pListElement, _T("list_item")));	
		list_item->SetBkImage(_T(""));
	}

	CTextUI* sign_center = static_cast<CTextUI*>(paint_manager_.FindSubControlByName(pListElement, _T("sign")));

	if(!item.folder && !item.sign.IsEmpty() && (0 == nDisplaType || 2 ==nDisplaType))
	{
		sign_center->SetVisible(true);

		_stprintf_s(szBuf, MAX_PATH - 1, _T("%s"), item.sign);
		sign_center->SetText(szBuf);
		sign_center->SetToolTip(szBuf);
	}
	else
	{
		sign_center->SetText(szBuf);
		sign_center->SetVisible(true);
		sign_center->SetToolTip(szBuf);
	}

	if (!item.folder && !item.description.IsEmpty())
	{
		CLabelUI* description = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(pListElement, kDescriptionControlName));

		const IGTBuddyUser * pBuddyUser = g_pMessengerActor->GetBuddyInfo(item.id);
		if(pBuddyUser == NULL)
		{
			pBuddyUser = g_pMessengerActor->GetBuddyInfoFromPersist(item.id);
		}
		ASSERT(pBuddyUser);
		if (pBuddyUser)
		{
			const NE_MEMBER_DATA* data = g_pMessengerActor->GetEmployeeInfo(item.id);
			GTUserStatusType stStatus = pBuddyUser->GetStatus();
			data = g_pMessengerActor->GetEmployeeInfo(item.id);
			stStatus = pBuddyUser->GetStatus();
			CDuiString strStatus;
			if (ustOFFLINE == stStatus && GET_CONFIG()->EnableFunction(_T("UI_01")) && !data->strWorkStatus.empty())
			{
				strTooltip = data->strWorkStatus.c_str();
				/*CString strToolTip = pMembereData->GetMemberData().strWorkStatus.c_str();*/
				strTooltip.Replace(_T(":"),_T("\n"));

			}

			if (!item.folder && !item.description.IsEmpty())
			{
#if defined(UNDER_WINCE)
				_stprintf(szBuf, _T("%s"), item.logo);
#else
				_stprintf_s(szBuf, MAX_PATH - 1, _T("%s"), item.description);
#endif
				description->SetBkImage(szBuf);
				description->SetToolTip(strTooltip);
			}

		}
	}

	CButtonUI* IPTBtn = static_cast<CButtonUI*>(paint_manager_.FindSubControlByName(pListElement, _T("iptbtn")));
	IPTBtn->SetTag((UINT_PTR)pListElement);

	IPTBtn->OnEvent += MakeDelegate(OnLogoButtonEvent);

	if(GET_APP()->IsAvailableIPT())
	{
		if (!item.folder && !item.IPTStatus.IsEmpty())
		{
			IPTBtn->SetVisible(true);
			const IGTBuddyUser * pBuddyUser = g_pMessengerActor->GetBuddyInfo(item.id);
			if(pBuddyUser == NULL)
			{
				pBuddyUser = g_pMessengerActor->GetBuddyInfoFromPersist(item.id);
			}
            ASSERT(pBuddyUser);
			if (pBuddyUser)
			{
				NTIPTStatusType stStatus = pBuddyUser->GetIPTStatus();

				CDuiString strStatus = GetIPTPresenceTip(stStatus);
				IPTBtn->SetToolTip(strStatus.GetData());
			}

			_stprintf_s(szBuf, MAX_PATH - 1, _T("%s"), item.IPTStatus);

			if(GET_APP()->IsAvailableIPT())
			{
				IPTBtn->SetBkImage(szBuf);
			}

		}
		else
		{
			IPTBtn->SetVisible(false);
		}
	}
	else
	{
		IPTBtn->SetVisible(false);
	}

	pListElement->SetFixedHeight(m_nItemNormalHeight);
	if(item.folder)pListElement->SetFixedHeight(33);
	pListElement->SetTag((UINT_PTR)node);
	int index = 0;
	int innerIndex = 0;
	int iSumChildCount = 0;
	if (parent->has_children())
	{
		Node* prev = parent->get_last_child();
		index = prev->data().list_elment_->GetIndex() + 1;
		for (int i = 0; i < parent->num_children(); i++)
		{
			Node* pNode = parent->child(i);
			if(pNode->GetCompareData() > nCompareData)
			{
				index = pNode->data().list_elment_->GetIndex();
				break;
			}
			else
			{
				iSumChildCount += pNode->num_children();
			}
		}

		if (parent == root_node_)
		{
			innerIndex = index - iSumChildCount;
		}
		else
		{
			innerIndex = index - parent->data().list_elment_->GetIndex() - 1;
		}

	}
	else 
	{
		if (parent == root_node_)
			index = 0;
		else
		{
			index = parent->data().list_elment_->GetIndex() + 1;
		}
		innerIndex = 0;
	}
	if (!CListUI::AddAt(pListElement, index))
	{
		delete pListElement;
		delete node;
		node = NULL;
	}

	parent->insert_child(node, innerIndex);
	return node;
}





bool CFriendsUI::RemoveNode(Node* node)
{

	/*GetList()->SetDelayedDestroy(false);
	SetDelayedDestroy(false);*/
	if (!node || node == root_node_) return false;


	int nCount = node->num_children();
	for (int i = nCount - 1; i >=0;  i--)
	{
		Node* child = node->child(i);
		RemoveNode(child);
	}

	CListUI::Remove(node->data().list_elment_);
	node->parent()->remove_child(node);
	delete node;
	node = NULL;

	return true;
}

void CFriendsUI::SetNodeEditable(Node* node, bool bEdit)
{
	if (!node || node == root_node_)
		return;

	CEditUI* edit_groupname = static_cast<CEditUI*>(paint_manager_.FindSubControlByName(node->data().list_elment_, _T("edit_groupname")));
	CLabelUI* nick_name = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(node->data().list_elment_, kNickNameControlName));

	if(nick_name != NULL && edit_groupname != NULL)
	{
		CString strNickName  = node->data().text_;
        CString strEditText = edit_groupname->GetText();
        strEditText.Trim();

		if(!bEdit)
		{
			CString oldName;
            //nick_name->SetText(strEditText);
			IGTBuddyGroup* pGroup = (IGTBuddyGroup*)node->GetNodeUserData();
            if (pGroup)
            {
                oldName = pGroup->GetName();
                if(strEditText != oldName)
                {
                    GET_MESSENGER_DLG()->ProcRenameGroupName(pGroup->GetID(), strEditText);
                }
                edit_groupname->SetVisible(false);
            }
		}
		else
		{
			IGTBuddyGroup* pGroup = (IGTBuddyGroup*)node->GetNodeUserData();
            if (pGroup)
            {
                edit_groupname->SetText(pGroup->GetName());
                edit_groupname->SetTag((UINT_PTR)node);
                edit_groupname->SetVisible(true);
                edit_groupname->SetFocus();
                edit_groupname->SetSelAll();
            }
		}

		nick_name->SetVisible(!bEdit);

	}
}

void CFriendsUI::SetChildVisible(Node* node, bool visible)
{
	CDuiString html_text = _T("");
	if (!node || node == root_node_)
		return;

	if(node->data().child_visible_ == visible)
	{
		return;
	}

	node->data().child_visible_ = visible;
	
	TCHAR szBuf[MAX_PATH] = {0};
	
	if (node->data().has_child_)
	{
#if defined(UNDER_WINCE)
		_stprintf(szBuf, _T("<x %d>"), 10);
#else
		_stprintf_s(szBuf, MAX_PATH - 1, _T("<x %d>"), 1);
#endif
		html_text += szBuf;
		html_text += node->data().text_;

		CLabelUI* arrow_icon = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(node->data().list_elment_, kArrowIconControlName));
		if (arrow_icon != NULL)
		{
			if (node->data().child_visible_)
			{
				if(!node->data().selected_)
				{
					arrow_icon->SetBkImage(_T("img_main_favorite_arrowup_nor.png"));
				}
				else
				{
					arrow_icon->SetBkImage(_T("img_main_favorite_arrowup_sel.png"));
				}
			}
			else
			{
				if(!node->data().selected_)
				{
					arrow_icon->SetBkImage(_T("img_main_favorite_arrowdown_nor.png"));
				}
				else
				{
					arrow_icon->SetBkImage(_T("img_main_favorite_arrowdown_sel.png"));
				}
			}
		}

		CLabelUI* nick_name = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(node->data().list_elment_, kNickNameControlName));
		if (nick_name != NULL)
		{
			nick_name->SetShowHtml(true);
			nick_name->SetText(html_text);
			nick_name->NeedParentUpdate();
		}
	}

	/*if (!node->data().list_elment_->IsVisible())
		return;*/

	if (!node->has_children())
		return;

	Node* begin = node->child(0);
	Node* end = node->get_last_child();
	int nDelayMs = 0;
    int step = 1;
	int ChildCnt = end->data().list_elment_->GetIndex() - begin->data().list_elment_->GetIndex()  +1;
	
	if (ChildCnt <= 8 && ChildCnt > 0){
		nDelayMs =  10;
		step = ChildCnt/5 + 1;
	}
	else if (ChildCnt <  20 && ChildCnt > 8)
	{
		nDelayMs = 10;
		step =ChildCnt/5 + 1;
	}
	else 
	{
		nDelayMs =  10; 
		step = ChildCnt/5 + 1;
	}
		

    if (visible == false)
    {
		for (int i = end->data().list_elment_->GetIndex(); i >= begin->data().list_elment_->GetIndex(); --i)
		{
			if (nDelayMs > 0)
			{
				if ((i+1 - begin->data().list_elment_->GetIndex())%step == 0)
				{
					/*CString str;
					str.Format(_T("sleep:%d\n"),i+1 - begin->data().list_elment_->GetIndex());
					OutputDebugString(str);*/
					Sleep(nDelayMs);

				}

			} 

			CControlUI* control = NULL;	

			control = GetItemAt(i);		

			if (_tcsicmp(control->GetClass(), _T("ListContainerElementUI")) == 0)
			{
				
				
				Node* local_parent = ((Node*)control->GetTag())->parent();

				control->SetVisible(false);	
				if (nDelayMs > 0)
				{
					if ((i+1 - begin->data().list_elment_->GetIndex())%step == 0)
					{
						UpdateWindow(GET_MESSENGER_DLG()->GetMainFrame()->GetSafeHwnd());

					}

				} 

				
			}
		}
    } 
    else
    {
		for (int i = end->data().list_elment_->GetIndex(); i >= begin->data().list_elment_->GetIndex(); --i)
		{
			if (nDelayMs > 0)
			{
				if ((i+1 - begin->data().list_elment_->GetIndex())%step == 0)
				{
					/*CString str;
					str.Format(_T("sleep:%d\n"),i+1 - begin->data().list_elment_->GetIndex());
					OutputDebugString(str);*/
					Sleep(nDelayMs);

				}

			} 

			CControlUI* control = NULL;	

			control = GetItemAt(i);		

			if (_tcsicmp(control->GetClass(), _T("ListContainerElementUI")) == 0)
			{
				

				Node* local_parent = ((Node*)control->GetTag())->parent();
				if (local_parent->data().child_visible_)
				{
					control->SetVisible(true);
					if (nDelayMs > 0)
					{
						if ((i+1 - begin->data().list_elment_->GetIndex())%step == 0)
						{
							control->Invalidate(TRUE);

						}

					} 
				}
				
			
			}
		}
    }
	
}

bool CFriendsUI::CanExpand(Node* node) const
{
	if (!node || node == root_node_)
		return false;

	return node->data().has_child_;
}

list<Node*> CFriendsUI::GetSelectedItems()
{
	list<Node*> listRet;
	list<int> listSelectedIndexs = CListUI::GetSelectedItems();
	for(list<int>::iterator it = listSelectedIndexs.begin();
		it != listSelectedIndexs.end(); it++)
	{
		CControlUI* pControl = GetItemAt(*it);
		//Node* node = NULL;
		if( pControl != NULL) 
		{
			Node* node = (Node*)pControl->GetTag();
			if(node != NULL)
			{
				listRet.push_back(node);
			}
		}
	}
	
	return listRet;
}

list<Node*> CFriendsUI::GetAllItems()
{
	list<Node*> listRet;

	listRet.clear();

	for(int i = 0; i < GetCount(); i++)
	{
		CControlUI* pControl = GetItemAt(i);

		Node* node = NULL;

		if( pControl != NULL) 
		{
			node = (Node*)pControl->GetTag();

			if(node != NULL)
			{
				listRet.push_back(node);
			}
		}
	}

	return listRet;
}

void CFriendsUI::SetSelectItemSize(int nWidth, int nHeight)
{
	m_nItemSelectedWidth = nWidth;
	m_nItemSelectedHeight = nHeight;
}

void CFriendsUI::SetNormalItemSize(int nWidth, int nHeight)
{
	m_nItemNormalWidth = nWidth;
	m_nItemNormalHeight = nHeight;
}

void CFriendsUI::SetSelectViewModal( CControlUI* pControl, bool bSelect)
{
	CListContainerElementUI* pFriendListItem = static_cast<CListContainerElementUI*>(pControl);
	Node* node = (Node*)pControl->GetTag();

	node->data().selected_ = bSelect;
	if (node->data().has_child_)
	{
		CLabelUI* arrow_icon = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(node->data().list_elment_, kArrowIconControlName));
		if (arrow_icon != NULL)
		{
			if (node->data().child_visible_)
			{
				if(!node->data().selected_)
				{
					arrow_icon->SetBkImage(_T("img_main_favorite_arrowup_nor.png"));
				}
				else
				{
					arrow_icon->SetBkImage(_T("img_main_favorite_arrowup_sel.png"));
				}
			}
			else
			{
				if(!node->data().selected_)
				{
					arrow_icon->SetBkImage(_T("img_main_favorite_arrowdown_nor.png"));
				}
				else
				{
					arrow_icon->SetBkImage(_T("img_main_favorite_arrowdown_sel.png"));
				}
			}
		}
	}
	SetChildVisible(node,node->data().child_visible_);

	if ((pFriendListItem != NULL) && (node != NULL) && !node->folder())
	{
		CContainerUI* pOperatorPannel = static_cast<CContainerUI*>(m_pManager->FindSubControlByName(pFriendListItem, kOperatorPannelControlName));
		if (pOperatorPannel != NULL)
		{
			pOperatorPannel->SetVisible(bSelect);
		}
	}
}





} // namespace DuiLib